@@ -39,9 +39,15 @@ typedef int error_t;
 
 #ifndef __P
 # if (defined __STDC__ && __STDC__) || defined __cplusplus
-#  define __P(args)	args
+#  if defined __GNUC__ && defined __cplusplus && __GNUC_MINOR__ >= 8
+#   define __P(args)	args throw ()
+#  else
+#   define __P(args)	args
+#  endif
+#  define __PMT(args)	args
 # else
 #  define __P(args)	()
+#  define __PMT(args)	()
 # endif
 #endif
 
@@ -124,7 +130,8 @@ struct argp_state;		/* " */
 struct argp_child;		/* " */
 
 /* The type of a pointer to an argp parsing function.  */
-typedef error_t (*argp_parser_t)(int key, char *arg, struct argp_state *state);
+typedef error_t (*argp_parser_t) __PMT ((int key, char *arg,
+					 struct argp_state *state));
 
 /* What to return for unrecognized keys.  For special ARGP_KEY_ keys, such
    returns will simply be ignored.  For user keys, this error will be turned
@@ -237,7 +244,8 @@ struct argp
      has been done, so if any of the replacement text also needs translation,
      that should be done by the filter function.  INPUT is either the input
      supplied to argp_parse, or NULL, if argp_help was called directly.  */
-  char *(*help_filter)(int __key, __const char *__text, void *__input);
+  char *(*help_filter) __PMT ((int __key, __const char *__text,
+			       void *__input));
 
   /* If non-zero the strings used in the argp library are translated using
      the domain described by this string.  Otherwise the currently installed
@@ -400,8 +408,8 @@ extern __const char *argp_program_version;
    calls this function with a stream to print the version to and a pointer to
    the current parsing state, and then exits (unless the ARGP_NO_EXIT flag is
    used).  This variable takes precedent over ARGP_PROGRAM_VERSION.  */
-extern void (*argp_program_version_hook) __P ((FILE *__stream,
-					       struct argp_state *__state));
+extern void (*argp_program_version_hook) __PMT ((FILE *__stream,
+						 struct argp_state *__state));
 
 /* If defined or set by the user program, it should point to string that is
    the bug-reporting address for the program.  It will be printed by
@@ -513,7 +521,7 @@ extern void *_argp_input __P ((__const struct argp *argp,
 extern void *__argp_input __P ((__const struct argp *argp,
 				__const struct argp_state *state));
 
-#ifdef __OPTIMIZE__
+#ifdef __USE_EXTERN_INLINES
 
 # if !_LIBC
 #  define __argp_usage argp_usage
@@ -523,17 +531,17 @@ extern void *__argp_input __P ((__const struct argp *argp,
 # endif
 
 # ifndef ARGP_EI
-#  define ARGP_EI extern inline
+#  define ARGP_EI extern __inline__
 # endif
 
 ARGP_EI void
-__argp_usage (__const struct argp_state *__state)
+__argp_usage (__const struct argp_state *__state) __THROW
 {
   __argp_state_help (__state, stderr, ARGP_HELP_STD_USAGE);
 }
 
 ARGP_EI int
-__option_is_short (__const struct argp_option *__opt)
+__option_is_short (__const struct argp_option *__opt) __THROW
 {
   if (__opt->flags & OPTION_DOC)
     return 0;
@@ -545,7 +553,7 @@ __option_is_short (__const struct argp_option *__opt)
 }
 
 ARGP_EI int
-__option_is_end (__const struct argp_option *__opt)
+__option_is_end (__const struct argp_option *__opt) __THROW
 {
   return !__opt->key && !__opt->name && !__opt->doc && !__opt->group;
 }
@@ -556,7 +564,7 @@ __option_is_end (__const struct argp_option *__opt)
 #  undef __option_is_short
 #  undef __option_is_end
 # endif
-#endif /* __OPTIMIZE__ */
+#endif /* Use extern inlines.  */
 
 #ifdef  __cplusplus
 }
