@@ -23,6 +23,7 @@
 #include <string.h>
 #include <bits/libc-lock.h>
 
+#include <ldsodefs.h>
 #include <gconv_int.h>
 
 
@@ -102,15 +103,15 @@ derivation_lookup (const char *fromset, const char *toset,
 		   struct gconv_step **handle, size_t *nsteps)
 {
   struct known_derivation key = { fromset, toset, NULL, 0 };
-  struct known_derivation *result;
+  struct known_derivation **result;
 
   result = __tfind (&key, &known_derivations, derivation_compare);
 
   if (result == NULL)
     return GCONV_NOCONV;
 
-  *handle = result->steps;
-  *nsteps = result->nsteps;
+  *handle = (*result)->steps;
+  *nsteps = (*result)->nsteps;
 
   /* Please note that we return GCONV_OK even if the last search for
      this transformation was unsuccessful.  */
@@ -146,7 +147,6 @@ add_derivation (const char *fromset, const char *toset,
 }
 
 static void
-internal_function
 free_derivation (void *p)
 {
   struct known_derivation *deriv = (struct known_derivation *) p;
@@ -154,7 +154,7 @@ free_derivation (void *p)
 
   for (cnt = 0; cnt < deriv->nsteps; ++cnt)
     if (deriv->steps[cnt].end_fct)
-      (*deriv->steps[cnt].end_fct) (&deriv->steps[cnt]);
+      _CALL_DL_FCT (deriv->steps[cnt].end_fct, (&deriv->steps[cnt]));
 
   free ((struct gconv_step *) deriv->steps);
   free (deriv);
@@ -192,6 +192,7 @@ gen_steps (struct derivation_step *best, const char *toset,
 				      ? __strdup (current->result_set)
 				      : result[step_cnt + 1].from_name);
 
+#ifndef STATIC_GCONV
 	  if (current->code->module_name[0] == '/')
 	    {
 	      /* Load the module, return handle for it.  */
@@ -212,13 +213,14 @@ gen_steps (struct derivation_step *best, const char *toset,
 	      result[step_cnt].end_fct = shlib_handle->end_fct;
 	    }
 	  else
+#endif
 	    /* It's a builtin transformation.  */
 	    __gconv_get_builtin_trans (current->code->module_name,
 				       &result[step_cnt]);
 
 	  /* Call the init function.  */
 	  if (result[step_cnt].init_fct != NULL)
-	    (*result[step_cnt].init_fct) (&result[step_cnt]);
+	    _CALL_DL_FCT (result[step_cnt].init_fct, (&result[step_cnt]));
 
 	  current = current->last;
 	}
@@ -229,11 +231,14 @@ gen_steps (struct derivation_step *best, const char *toset,
 	  while (++step_cnt < *nsteps)
 	    {
 	      if (result[step_cnt].end_fct != NULL)
-		(*result[step_cnt].end_fct) (&result[step_cnt]);
+		_CALL_DL_FCT (result[step_cnt].end_fct, (&result[step_cnt]));
+#ifndef STATIC_GCONV
 	      __gconv_release_shlib (result[step_cnt].shlib_handle);
+#endif
 	    }
 	  free (result);
 	  *nsteps = 0;
+	  *handle = NULL;
 	  status = GCONV_NOCONV;
 	}
       else
@@ -242,6 +247,11 @@ gen_steps (struct derivation_step *best, const char *toset,
 	  status = GCONV_OK;
 	}
     }
+  else
+    {
+      *nsteps = 0;
+      *handle = NULL;
+    }
 
   return status;
 }
@@ -257,7 +267,8 @@ find_derivation (const char *toset, const char *toset_expand,
 {
   __libc_lock_define_initialized (static, lock)
   struct derivation_step *first, *current, **lastp, *best = NULL;
-  int best_cost = 0;
+  int best_cost_hi = 0;
+  int best_cost_lo = 0;
   int result;
 
   result = derivation_lookup (fromset_expand ?: fromset, toset_expand ?: toset,
@@ -425,18 +436,22 @@ find_derivation (const char *toset, const char *toset_expand,
 		  /* Determine the costs.  If they are lower than the
 		     previous solution (or this is the first solution)
 		     remember this solution.  */
-		  int cost = __gconv_modules_db[cnt]->cost;
+		  int cost_hi = __gconv_modules_db[cnt]->cost_hi;
+		  int cost_lo = __gconv_modules_db[cnt]->cost_lo;
 		  struct derivation_step *runp = current;
 		  while (runp->code != NULL)
 		    {
-		      cost += runp->code->cost;
+		      cost_hi += runp->code->cost_hi;
+		      cost_lo += runp->code->cost_lo;
 		      runp = runp->last;
 		    }
-		  if (best == NULL || cost < best_cost)
+		  if (best == NULL || cost_hi < best_cost_hi
+		      || (cost_hi == best_cost_hi && cost_lo < best_cost_lo))
 		    {
 		      best = NEW_STEP (result_set, __gconv_modules_db[cnt],
 				       current);
-		      best_cost = cost;
+		      best_cost_hi = cost_hi;
+		      best_cost_lo = cost_lo;
 		    }
 		}
 	      else
@@ -525,6 +540,7 @@ __gconv_find_transform (const char *toset, const char *fromset,
   result = find_derivation (toset, toset_expand, fromset, fromset_expand,
 			    handle, nsteps);
 
+#ifndef STATIC_GCONV
   /* Increment the user counter.  */
   if (result == GCONV_OK)
     {
@@ -548,6 +564,7 @@ __gconv_find_transform (const char *toset, const char *fromset,
 	  }
       while (cnt > 0);
     }
+#endif
 
   /* Release the lock.  */
   __libc_lock_unlock (lock);
@@ -568,6 +585,7 @@ __gconv_close_transform (struct gconv_step *steps, size_t nsteps)
 {
   int result = GCONV_OK;
 
+#ifndef STATIC_GCONV
   /* Acquire the lock.  */
   __libc_lock_lock (lock);
 
@@ -583,6 +601,7 @@ __gconv_close_transform (struct gconv_step *steps, size_t nsteps)
 
   /* Release the lock.  */
   __libc_lock_unlock (lock);
+#endif
 
   return result;
 }
