@@ -135,6 +135,9 @@ ctype_startup (struct linereader *lr, struct localedef_t *locale,
   unsigned int cnt;
   struct locale_ctype_t *ctype;
 
+  /* We have a definition for LC_CTYPE.  */
+  copy_posix.mask &= ~(1 << LC_CTYPE);
+
   /* It is important that we always use UCS1 encoding for strings now.  */
   encoding_method = ENC_UCS1;
 
@@ -308,7 +311,9 @@ character %s'%s' in class `%s' must not be in class `%s'"),
     }
 
   /* ... and now test <SP> as a special case.  */
-  space_value = charset_find_value (charset, "SP", 2);
+  space_value = charset_find_value (&charset->char_table, "SP", 2);
+  if ((wchar_t) space_value == ILLEGAL_CHAR_VALUE)
+    space_value = charset_find_value (&charset->char_table, "space", 5);
   if ((wchar_t) space_value == ILLEGAL_CHAR_VALUE)
     {
       if (!be_quiet)
@@ -634,7 +639,8 @@ ctype_class_from (struct linereader *lr, struct localedef_t *locale,
   struct locale_ctype_t *ctype = locale->categories[LC_CTYPE].ctype;
   unsigned int value;
 
-  value = charset_find_value (charset, code->val.str.start, code->val.str.len);
+  value = charset_find_value (&charset->char_table, code->val.str.start,
+			      code->val.str.len);
 
   ctype->last_class_char = value;
 
@@ -656,7 +662,8 @@ ctype_class_to (struct linereader *lr, struct localedef_t *locale,
   struct locale_ctype_t *ctype = locale->categories[LC_CTYPE].ctype;
   unsigned int value, cnt;
 
-  value = charset_find_value (charset, code->val.str.start, code->val.str.len);
+  value = charset_find_value (&charset->char_table, code->val.str.start,
+			      code->val.str.len);
 
   /* In the LC_CTYPE category it is no error when a character is
      not found.  This has to be ignored silently.  */
@@ -750,7 +757,8 @@ ctype_map_from (struct linereader *lr, struct localedef_t *locale,
   struct locale_ctype_t *ctype = locale->categories[LC_CTYPE].ctype;
   unsigned int value;
 
-  value = charset_find_value (charset, code->val.str.start, code->val.str.len);
+  value = charset_find_value (&charset->char_table, code->val.str.start,
+			      code->val.str.len);
 
   if ((wchar_t) value == ILLEGAL_CHAR_VALUE)
     /* In the LC_CTYPE category it is no error when a character is
@@ -770,7 +778,8 @@ ctype_map_to (struct linereader *lr, struct localedef_t *locale,
   struct locale_ctype_t *ctype = locale->categories[LC_CTYPE].ctype;
   unsigned int value;
 
-  value = charset_find_value (charset, code->val.str.start, code->val.str.len);
+  value = charset_find_value (&charset->char_table, code->val.str.start,
+			      code->val.str.len);
 
   if ((wchar_t) ctype->from_map_char == ILLEGAL_CHAR_VALUE
       || (wchar_t) value == ILLEGAL_CHAR_VALUE)
@@ -948,7 +957,7 @@ set_class_defaults (struct locale_ctype_t *ctype, struct charset_t *charset)
 	  unsigned int value;
 	  tmp[0] = ch;
 
-	  value = charset_find_value (charset, tmp, 1);
+	  value = charset_find_value (&charset->char_table, tmp, 1);
 	  if ((wchar_t) value == ILLEGAL_CHAR_VALUE)
 	    {
 	      if (!be_quiet)
@@ -1013,7 +1022,7 @@ character `%s' not defined while needed as default value"),
     {
       unsigned int value;
 
-      value = charset_find_value (charset, "space", 5);
+      value = charset_find_value (&charset->char_table, "space", 5);
       if ((wchar_t) value == ILLEGAL_CHAR_VALUE)
 	{
 	  if (!be_quiet)
@@ -1024,7 +1033,7 @@ character `%s' not defined while needed as default value"),
       else
 	ELEM (ctype, class_collection, , value) |= BIT (tok_space);
 
-      value = charset_find_value (charset, "form-feed", 9);
+      value = charset_find_value (&charset->char_table, "form-feed", 9);
       if ((wchar_t) value == ILLEGAL_CHAR_VALUE)
 	{
 	  if (!be_quiet)
@@ -1035,7 +1044,7 @@ character `%s' not defined while needed as default value"),
       else
 	ELEM (ctype, class_collection, , value) |= BIT (tok_space);
 
-      value = charset_find_value (charset, "newline", 7);
+      value = charset_find_value (&charset->char_table, "newline", 7);
       if ((wchar_t) value == ILLEGAL_CHAR_VALUE)
 	{
 	  if (!be_quiet)
@@ -1046,7 +1055,7 @@ character `%s' not defined while needed as default value"),
       else
 	ELEM (ctype, class_collection, , value) |= BIT (tok_space);
 
-      value = charset_find_value (charset, "carriage-return", 15);
+      value = charset_find_value (&charset->char_table, "carriage-return", 15);
       if ((wchar_t) value == ILLEGAL_CHAR_VALUE)
 	{
 	  if (!be_quiet)
@@ -1057,7 +1066,7 @@ character `%s' not defined while needed as default value"),
       else
 	ELEM (ctype, class_collection, , value) |= BIT (tok_space);
 
-      value = charset_find_value (charset, "tab", 3);
+      value = charset_find_value (&charset->char_table, "tab", 3);
       if ((wchar_t) value == ILLEGAL_CHAR_VALUE)
 	{
 	  if (!be_quiet)
@@ -1068,7 +1077,7 @@ character `%s' not defined while needed as default value"),
       else
 	ELEM (ctype, class_collection, , value) |= BIT (tok_space);
 
-      value = charset_find_value (charset, "vertical-tab", 12);
+      value = charset_find_value (&charset->char_table, "vertical-tab", 12);
       if ((wchar_t) value == ILLEGAL_CHAR_VALUE)
 	{
 	  if (!be_quiet)
@@ -1097,7 +1106,7 @@ character `%s' not defined while needed as default value"),
    {
       unsigned int value;
 
-      value = charset_find_value (charset, "space", 5);
+      value = charset_find_value (&charset->char_table, "space", 5);
       if ((wchar_t) value == ILLEGAL_CHAR_VALUE)
 	{
 	  if (!be_quiet)
@@ -1108,7 +1117,7 @@ character `%s' not defined while needed as default value"),
       else
 	ELEM (ctype, class_collection, , value) |= BIT (tok_blank);
 
-      value = charset_find_value (charset, "tab", 3);
+      value = charset_find_value (&charset->char_table, "tab", 3);
       if ((wchar_t) value == ILLEGAL_CHAR_VALUE)
 	{
 	  if (!be_quiet)
@@ -1149,7 +1158,7 @@ character `%s' not defined while needed as default value"),
 	if ((ctype->class_collection[cnt] & mask) != 0)
 	  ctype->class_collection[cnt] |= BIT (tok_print);
 
-      space = charset_find_value (charset, "space", 5);
+      space = charset_find_value (&charset->char_table, "space", 5);
       if (space == ILLEGAL_CHAR_VALUE)
 	{
 	  if (!be_quiet)
@@ -1178,7 +1187,7 @@ character `%s' not defined while needed as default value"),
 
 	  tmp[1] = (char) ch;
 
-	  value_from = charset_find_value (charset, &tmp[1], 1);
+	  value_from = charset_find_value (&charset->char_table, &tmp[1], 1);
 	  if ((wchar_t) value_from == ILLEGAL_CHAR_VALUE)
 	    {
 	      if (!be_quiet)
@@ -1190,7 +1199,7 @@ character `%s' not defined while needed as default value"),
 
 	  /* This conversion is implementation defined.  */
 	  tmp[1] = (char) (ch + ('A' - 'a'));
-	  value_to = charset_find_value (charset, &tmp[1], 1);
+	  value_to = charset_find_value (&charset->char_table, &tmp[1], 1);
 	  if ((wchar_t) value_to == ILLEGAL_CHAR_VALUE)
 	    {
 	      if (!be_quiet)
@@ -1443,11 +1452,8 @@ Computing table size for character classes might take a while..."),
 	    }
     }
 
-  /* Compute MB_CUR_MAX.  Please note the value mb_cur_max in the
-     character set definition gives the number of bytes in the wide
-     character representation.  We compute the number of bytes used
-     for the UTF-8 encoded form.  */
-  ctype->mb_cur_max = ((int []) { 2, 3, 5, 6 }) [charset->mb_cur_max - 1];
+  /* Compute MB_CUR_MAX.  */
+  ctype->mb_cur_max = charset->mb_cur_max;
 
   /* We need the name of the currently used 8-bit character set to
      make correct conversion between this 8-bit representation and the
