@@ -26,7 +26,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <elf/ldsodefs.h>	/* We need some help from ld.so.  */
+#include <ldsodefs.h>		/* We need some help from ld.so.  */
 
 #if !defined DO_STATIC_NSS || defined PIC
 # include <gnu/lib-names.h>
@@ -35,8 +35,6 @@
 #include "nsswitch.h"
 
 /* Prototypes for the local functions.  */
-static void *nss_lookup_function (service_user *ni, const char *fct_name)
-     internal_function;
 static name_database *nss_parse_file (const char *fname) internal_function;
 static name_database_entry *nss_getline (char *line) internal_function;
 static service_user *nss_parse_service_list (const char *line)
@@ -140,7 +138,7 @@ __nss_database_lookup (const char *database, const char *alternate_name,
 int
 __nss_lookup (service_user **ni, const char *fct_name, void **fctp)
 {
-  *fctp = nss_lookup_function (*ni, fct_name);
+  *fctp = __nss_lookup_function (*ni, fct_name);
 
   while (*fctp == NULL
 	 && nss_next_action (*ni, NSS_STATUS_UNAVAIL) == NSS_ACTION_CONTINUE
@@ -148,7 +146,7 @@ __nss_lookup (service_user **ni, const char *fct_name, void **fctp)
     {
       *ni = (*ni)->next;
 
-      *fctp = nss_lookup_function (*ni, fct_name);
+      *fctp = __nss_lookup_function (*ni, fct_name);
     }
 
   return *fctp != NULL ? 0 : (*ni)->next == NULL ? 1 : -1;
@@ -187,7 +185,7 @@ __nss_next (service_user **ni, const char *fct_name, void **fctp, int status,
     {
       *ni = (*ni)->next;
 
-      *fctp = nss_lookup_function (*ni, fct_name);
+      *fctp = __nss_lookup_function (*ni, fct_name);
     }
   while (*fctp == NULL
 	 && nss_next_action (*ni, NSS_STATUS_UNAVAIL) == NSS_ACTION_CONTINUE
@@ -294,10 +292,10 @@ static void
 get_sym (void *a)
 {
   struct get_sym_args *args = (struct get_sym_args *) a;
-  struct link_map *scope[2] = { args->map, NULL };
   args->ref = NULL;
   args->loadbase = _dl_lookup_symbol (args->name, &args->ref,
-				      scope, args->map->l_name, 0);
+				      args->map->l_local_scope,
+				      args->map->l_name, 0);
 }
 #endif
 
@@ -310,9 +308,8 @@ known_compare (const void *p1, const void *p2)
 }
 
 
-static void *
-internal_function
-nss_lookup_function (service_user *ni, const char *fct_name)
+void *
+__nss_lookup_function (service_user *ni, const char *fct_name)
 {
   void **found, *result;
 
@@ -539,7 +536,7 @@ nss_parse_file (const char *fname)
 	  last = this;
 	}
     }
-  while (!feof (fp));
+  while (!feof_unlocked (fp));
 
   /* Free the buffer.  */
   free (line);
@@ -621,7 +618,8 @@ nss_parse_service_list (const char *line)
 	      lookup_actions action;
 
 	      /* Grok ! before name to mean all statii but that one.  */
-	      if (not = line[0] == '!')
+	      not = line[0] == '!';
+	      if (not)
 		++line;
 
 	      /* Read status name.  */
