@@ -21,6 +21,7 @@
 #include <ctype.h>
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <wctype.h>
@@ -46,8 +47,6 @@
 # define MALLOC		0x100	/* a: malloc strings */
 # define CHAR		0x200	/* hh: char */
 
-# define TYPEMOD	(LONG|LONGDBL|SHORT|CHAR)
-
 
 #ifdef USE_IN_LIBIO
 # include <libioP.h>
@@ -55,30 +54,35 @@
 
 # undef va_list
 # define va_list	_IO_va_list
-# define ungetc(c, s)	((void) ((int) c != EOF && --read_in),		      \
-			 _IO_ungetc (c, s))
+# define ungetc(c, s)	((void) ((int) c == EOF				      \
+				 || (--read_in,				      \
+				     _IO_sputbackc (s, (unsigned char) c))))
 # define inchar()	(c == EOF ? EOF					      \
 			 : ((c = _IO_getc_unlocked (s)),		      \
 			    (void) (c != EOF && ++read_in), c))
 # define encode_error()	do {						      \
 			  if (errp != NULL) *errp |= 4;			      \
 			  _IO_funlockfile (s);				      \
+			  __libc_cleanup_end (0);			      \
 			  __set_errno (EILSEQ);				      \
 			  return done;					      \
 			} while (0)
 # define conv_error()	do {						      \
 			  if (errp != NULL) *errp |= 2;			      \
 			  _IO_funlockfile (s);				      \
+			  __libc_cleanup_end (0);			      \
 			  return done;					      \
 			} while (0)
 # define input_error()	do {						      \
 			  _IO_funlockfile (s);				      \
 			  if (errp != NULL) *errp |= 1;			      \
+			  __libc_cleanup_end (0);			      \
 			  return done ?: EOF;				      \
 			} while (0)
 # define memory_error()	do {						      \
 			  _IO_funlockfile (s);				      \
 			  __set_errno (ENOMEM);				      \
+			  __libc_cleanup_end (0);			      \
 			  return EOF;					      \
 			} while (0)
 # define ARGCHECK(s, format)						      \
@@ -100,7 +104,9 @@
 # define LOCK_STREAM(S)							      \
   __libc_cleanup_region_start ((void (*) (void *)) &_IO_funlockfile, (S));    \
   _IO_flockfile (S)
-# define UNLOCK_STREAM __libc_cleanup_region_end (1)
+# define UNLOCK_STREAM(S)						      \
+  _IO_funlockfile (S);							      \
+  __libc_cleanup_region_end (0)
 #else
 # define ungetc(c, s)	((void) (c != EOF && --read_in), ungetc (c, s))
 # define inchar()	(c == EOF ? EOF					      \
@@ -143,12 +149,14 @@
 # define flockfile(S) /* nothing */
 # define funlockfile(S) /* nothing */
 # define LOCK_STREAM(S)
-# define UNLOCK_STREAM
+# define UNLOCK_STREAM(S)
 #else
 # define LOCK_STREAM(S)							      \
   __libc_cleanup_region_start (&__funlockfile, (S));			      \
   __flockfile (S)
-# define UNLOCK_STREAM __libc_cleanup_region_end (1)
+# define UNLOCK_STREAM(S)						      \
+  __funlockfile (S);							      \
+  __libc_cleanup_region_end (0)
 #endif
 #endif
 
@@ -208,6 +216,8 @@ __vfscanf (FILE *s, const char *format, va_list argptr)
      possibly be matched even if in the input stream no character is
      available anymore.  */
   int skip_space = 0;
+  /* Nonzero if we are reading a pointer.  */
+  int read_pointer;
   /* Workspace.  */
   char *tw;			/* Temporary pointer.  */
   char *wp = NULL;		/* Workspace.  */
@@ -346,6 +356,9 @@ __vfscanf (FILE *s, const char *format, va_list argptr)
       /* This is the start of the conversion string. */
       flags = 0;
 
+      /* Not yet decided whether we read a pointer or not.  */
+      read_pointer = 0;
+
       /* Initialize state of modifiers.  */
       argpos = 0;
 
@@ -398,59 +411,69 @@ __vfscanf (FILE *s, const char *format, va_list argptr)
 	width = -1;
 
       /* Check for type modifiers.  */
-      while (*f == 'h' || *f == 'l' || *f == 'L' || *f == 'a' || *f == 'q')
-	switch (*f++)
-	  {
-	  case 'h':
-	    /* int's are short int's.  */
-	    if (flags & (LONG|LONGDBL|CHAR))
-	      /* Signal illegal format element.  */
-	      conv_error ();
-	    if (flags & SHORT)
-	      {
-		flags &= ~SHORT;
-		flags |= CHAR;
-	      }
-	    else
-	      flags |= SHORT;
-	    break;
-	  case 'l':
-	    if (flags & (SHORT|LONGDBL|CHAR))
-	      conv_error ();
-	    else if (flags & LONG)
-	      {
-		/* A double `l' is equivalent to an `L'.  */
-		flags &= ~LONG;
-		flags |= LONGDBL;
-	      }
-	    else
-	      /* int's are long int's.  */
-	      flags |= LONG;
-	    break;
-	  case 'q':
-	  case 'L':
-	    /* double's are long double's, and int's are long long int's.  */
-	    if (flags & TYPEMOD)
-	      /* Signal illegal format element.  */
-	      conv_error ();
+      switch (*f++)
+	{
+	case 'h':
+	  /* ints are short ints or chars.  */
+	  if (*f == 'h')
+	    {
+	      ++f;
+	      flags |= CHAR;
+	    }
+	  else
+	    flags |= SHORT;
+	  break;
+	case 'l':
+	  if (*f == 'l')
+	    {
+	      /* A double `l' is equivalent to an `L'.  */
+	      ++f;
+	      flags |= LONGDBL;
+	    }
+	  else
+	    /* ints are long ints.  */
+	    flags |= LONG;
+	  break;
+	case 'q':
+	case 'L':
+	  /* doubles are long doubles, and ints are long long ints.  */
+	  flags |= LONGDBL;
+	  break;
+	case 'a':
+	  /* The `a' is used as a flag only if followed by `s', `S' or
+	     `['.  */
+	  if (*f != 's' && *f != 'S' && *f != '[')
+	    {
+	      --f;
+	      break;
+	    }
+	  /* String conversions (%s, %[) take a `char **'
+	     arg and fill it in with a malloc'd pointer.  */
+	  flags |= MALLOC;
+	  break;
+	case 'z':
+	  if (sizeof (size_t) > sizeof (unsigned long int))
 	    flags |= LONGDBL;
-	    break;
-	  case 'a':
-	    /* The `a' is used as a flag only if followed by `s', `S' or
-	       `['.  */
-	    if (*f != 's' && *f != 'S' && *f != '[')
-	      {
-		--f;
-		break;
-	      }
-	    if (flags & TYPEMOD)
-	      /* Signal illegal format element.  */
-	      conv_error ();
-	    /* String conversions (%s, %[) take a `char **'
-	       arg and fill it in with a malloc'd pointer.  */
-	    flags |= MALLOC;
-	    break;
-	  }
+	  else if (sizeof (size_t) > sizeof (unsigned int))
+	    flags |= LONG;
+	  break;
+	case 'j':
+	  if (sizeof (uintmax_t) > sizeof (unsigned long int))
+	    flags |= LONGDBL;
+	  else if (sizeof (uintmax_t) > sizeof (unsigned int))
+	    flags |= LONG;
+	  break;
+	case 't':
+	  if (sizeof (ptrdiff_t) > sizeof (long int))
+	    flags |= LONGDBL;
+	  else if (sizeof (ptrdiff_t) > sizeof (int))
+	    flags |= LONG;
+	  break;
+	default:
+	  /* Not a recognized modifier.  Backup.  */
+	  --f;
+	  break;
+	}
 
       /* End of the format string?  */
       if (*f == '\0')
@@ -584,11 +607,13 @@ __vfscanf (FILE *s, const char *format, va_list argptr)
 #define NEXT_WIDE_CHAR(First)						      \
 		c = inchar ();						      \
 		if (c == EOF)						      \
-		  /* EOF is only an error for the first character.  */	      \
-		  if (First)						      \
-		    input_error ();					      \
-		  else							      \
-		    break;						      \
+		  {							      \
+		    /* EOF is only an error for the first character.  */      \
+		    if (First)						      \
+		      input_error ();					      \
+		    else						      \
+		      break;						      \
+		  }							      \
 		val = c;						      \
 		if (val >= 0x80)					      \
 		  {							      \
@@ -851,13 +876,32 @@ __vfscanf (FILE *s, const char *format, va_list argptr)
 	      c = inchar ();
 	    }
 
-	  /* The just read character is not part of the number anymore.  */
-	  ungetc (c, s);
-
 	  if (wpsize == 0 ||
 	      (wpsize == 1 && (wp[0] == '+' || wp[0] == '-')))
-	    /* There was no number.  */
-	    conv_error ();
+	    {
+	      /* There was no number.  If we are supposed to read a pointer
+		 we must recognize "(nil)" as well.  */
+	      if (wpsize == 0 && read_pointer && (width < 0 || width >= 0)
+		  && c == '('
+		  && tolower (inchar ()) == 'n'
+		  && tolower (inchar ()) == 'i'
+		  && tolower (inchar ()) == 'l'
+		  && inchar () == ')')
+		/* We must produce the value of a NULL pointer.  A single
+		   '0' digit is enough.  */
+		ADDW ('0');
+	      else
+		{
+		  /* The last read character is not part of the number
+		     anymore.  */
+		  ungetc (c, s);
+
+		  conv_error ();
+		}
+	    }
+	  else
+	    /* The just read character is not part of the number anymore.  */
+	    ungetc (c, s);
 
 	  /* Convert the number.  */
 	  ADDW ('\0');
@@ -935,6 +979,57 @@ __vfscanf (FILE *s, const char *format, va_list argptr)
 	  else
 	    negative = 0;
 
+	  /* Take care for the special arguments "nan" and "inf".  */
+	  if (tolower (c) == 'n')
+	    {
+	      /* Maybe "nan".  */
+	      ADDW (c);
+	      if (inchar () == EOF || tolower (c) != 'a')
+		input_error ();
+	      ADDW (c);
+	      if (inchar () == EOF || tolower (c) != 'n')
+		input_error ();
+	      ADDW (c);
+	      /* It is "nan".  */
+	      goto scan_float;
+	    }
+	  else if (tolower (c) == 'i')
+	    {
+	      /* Maybe "inf" or "infinity".  */
+	      ADDW (c);
+	      if (inchar () == EOF || tolower (c) != 'n')
+		input_error ();
+	      ADDW (c);
+	      if (inchar () == EOF || tolower (c) != 'f')
+		input_error ();
+	      ADDW (c);
+	      /* It is as least "inf".  */
+	      if (inchar () != EOF)
+		{
+		  if (tolower (c) == 'i')
+		    {
+		      /* No we have to read the rest as well.  */
+		      ADDW (c);
+		      if (inchar () == EOF || tolower (c) != 'n')
+			input_error ();
+		      ADDW (c);
+		      if (inchar () == EOF || tolower (c) != 'i')
+			input_error ();
+		      ADDW (c);
+		      if (inchar () == EOF || tolower (c) != 't')
+			input_error ();
+		      ADDW (c);
+		      if (inchar () == EOF || tolower (c) != 'y')
+			input_error ();
+		      ADDW (c);
+		    }
+		  else
+		    /* Never mind.  */
+		    ungetc (c, s);
+		}
+	      goto scan_float;
+	    }
+
 	  is_hexa = 0;
 	  exp_char = 'e';
 	  if (c == '0')
@@ -995,6 +1090,7 @@ __vfscanf (FILE *s, const char *format, va_list argptr)
 	  if (wpsize == 0 || (is_hexa && wpsize == 2))
 	    conv_error ();
 
+	scan_float:
 	  /* Convert the number.  */
 	  ADDW ('\0');
 	  if (flags & LONGDBL)
@@ -1149,7 +1245,12 @@ __vfscanf (FILE *s, const char *format, va_list argptr)
 	  flags &= ~(SHORT|LONGDBL);
 	  flags |= LONG;
 	  number_signed = 0;
+	  read_pointer = 1;
 	  goto number;
+
+	default:
+	  /* If this is an unknown format character punt.  */
+	  conv_error ();
 	}
     }
 
@@ -1164,7 +1265,7 @@ __vfscanf (FILE *s, const char *format, va_list argptr)
     }
 
   /* Unlock stream.  */
-  UNLOCK_STREAM;
+  UNLOCK_STREAM (s);
 
   return done;
 }
