@@ -1,4 +1,4 @@
-/* Copyright (C) 1991, 92, 93, 94, 95, 96, 97 Free Software Foundation, Inc.
+/* Copyright (C) 1991,92,93,94,95,96,97,98 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -74,8 +74,8 @@ typedef struct
 
 /* Read NBYTES bytes from COOKIE into a buffer pointed to by BUF.
    Return number of bytes read.  */
-typedef __ssize_t __io_read_fn __P ((__ptr_t __cookie, char *__buf,
-				     size_t __nbytes));
+typedef __ssize_t __io_read_fn __PMT ((__ptr_t __cookie, char *__buf,
+				       size_t __nbytes));
 
 /* Write N bytes pointed to by BUF to COOKIE.  Write all N bytes
    unless there is an error.  Return number of bytes written, or -1 if
@@ -83,7 +83,7 @@ typedef __ssize_t __io_read_fn __P ((__ptr_t __cookie, char *__buf,
    opened for append (__mode.__append set), then set the file pointer
    to the end of the file and then do the write; if not, just write at
    the current file pointer.  */
-typedef __ssize_t __io_write_fn __P ((__ptr_t __cookie, __const char *__buf,
+typedef __ssize_t __io_write_fn __PMT ((__ptr_t __cookie, __const char *__buf,
 				      size_t __n));
 
 /* Move COOKIE's file position to *POS bytes from the
@@ -92,14 +92,14 @@ typedef __ssize_t __io_write_fn __P ((__ptr_t __cookie, __const char *__buf,
    or the end of the file (if W is SEEK_END).
    Set *POS to the new file position.
    Returns zero if successful, nonzero if not.  */
-typedef int __io_seek_fn __P ((__ptr_t __cookie, fpos_t *__pos, int __w));
+typedef int __io_seek_fn __PMT ((__ptr_t __cookie, fpos_t *__pos, int __w));
 
 /* Close COOKIE.  */
-typedef int __io_close_fn __P ((__ptr_t __cookie));
+typedef int __io_close_fn __PMT ((__ptr_t __cookie));
 
 /* Return the file descriptor associated with COOKIE,
    or -1 on error.  There need not be any associated file descriptor.  */
-typedef int __io_fileno_fn __P ((__ptr_t __cookie));
+typedef int __io_fileno_fn __PMT ((__ptr_t __cookie));
 
 #ifdef __USE_GNU
 /* User-visible names for the above.  */
@@ -139,9 +139,9 @@ typedef struct
 typedef struct
 {
   /* Make room in the input buffer.  */
-  int (*__input) __P ((FILE *__stream));
+  int (*__input) __PMT ((FILE *__stream));
   /* Make room in the output buffer.  */
-  void (*__output) __P ((FILE *__stream, int __c));
+  void (*__output) __PMT ((FILE *__stream, int __c));
 } __room_functions;
 
 extern __const __io_functions __default_io_functions;
@@ -155,20 +155,8 @@ extern int __stdio_open __P ((__const char *__file, __io_mode __m,
 			      __ptr_t *__cookieptr));
 /* Put out an error message for when stdio needs to die.  */
 extern void __stdio_errmsg __P ((__const char *__msg, size_t __len));
-/* Generate a unique file name (and possibly open it with mode "w+b").  */
-extern char *__stdio_gen_tempname __P ((char *__buf, size_t __bufsize,
-					__const char *__dir,
-					__const char *__pfx,
-					int __dir_search,
-					size_t *__lenptr,
-					FILE **__streamptr,
-					int __large_file));
 
 
-/* Print out MESSAGE on the error output and abort.  */
-extern void __libc_fatal __P ((__const char *__message))
-     __attribute__ ((__noreturn__));
-
 
 /* For thread safe I/O functions we need a lock in each stream.  We
    keep the type opaque here.  */
@@ -333,6 +321,11 @@ extern int fclose __P ((FILE *__stream));
 /* Flush STREAM, or all streams if STREAM is NULL.  */
 extern int fflush __P ((FILE *__stream));
 
+#ifdef __USE_MISC
+/* Faster versions when locking is not required.  */
+extern int fflush_unlocked __P ((FILE *__stream));
+#endif
+
 #ifdef __USE_GNU
 /* Close all streams.  */
 extern int __fcloseall __P ((void));
@@ -360,6 +353,7 @@ extern FILE *__newstream __P ((void));
 
 #ifdef	__USE_POSIX
 /* Create a new stream that refers to an existing system file descriptor.  */
+extern FILE *__fdopen __P ((int __fd, __const char *__modes));
 extern FILE *fdopen __P ((int __fd, __const char *__modes));
 #endif
 
@@ -421,7 +415,7 @@ extern int vsprintf __P ((char *__restrict __s,
 
 #ifdef	__OPTIMIZE__
 extern __inline int
-vprintf (const char *__restrict __fmt, __gnuc_va_list __arg)
+vprintf (const char *__restrict __fmt, __gnuc_va_list __arg) __THROW
 {
   return vfprintf (stdout, __fmt, __arg);
 }
@@ -447,18 +441,22 @@ extern int vsnprintf __P ((char *__s, size_t __maxlen,
 #ifdef __USE_GNU
 /* Write formatted output to a string dynamically allocated with `malloc'.
    Store the address of the string in *PTR.  */
-extern int vasprintf __P ((char **__ptr, __const char *__f,
-			   __gnuc_va_list __arg))
-     __attribute__ ((__format__ (__printf__, 2, 0)));
-extern int asprintf __P ((char **__ptr, __const char *__fmt, ...))
+extern int vasprintf __P ((char **__restrict __ptr,
+			   __const char *__restrict __f, __gnuc_va_list __arg))
      __attribute__ ((__format__ (__printf__, 2, 0)));
+extern int __asprintf __P ((char **__restrict __ptr,
+			    __const char *__restrict __fmt, ...))
+     __attribute__ ((__format__ (__printf__, 2, 3)));
+extern int asprintf __P ((char **__restrict __ptr,
+			  __const char *__restrict __fmt, ...))
+     __attribute__ ((__format__ (__printf__, 2, 3)));
 
 /* Write formatted output to a file descriptor.  */
-extern int vdprintf __P ((int __fd, __const char *__fmt,
+extern int vdprintf __P ((int __fd, __const char *__restrict __fmt,
 			  __gnuc_va_list __arg))
      __attribute__ ((__format__ (__printf__, 2, 0)));
-extern int dprintf __P ((int __fd, __const char *__fmt, ...))
-     __attribute__ ((__format__ (__printf__, 2, 0)));
+extern int dprintf __P ((int __fd, __const char *__restrict __fmt, ...))
+     __attribute__ ((__format__ (__printf__, 2, 3)));
 #endif
 
 
@@ -471,7 +469,7 @@ extern int scanf __P ((__const char *__restrict __format, ...));
 extern int sscanf __P ((__const char *__restrict __s,
 			__const char *__restrict __format, ...));
 
-#ifdef	__USE_GNU
+#ifdef	__USE_ISOC9X
 /* Read formatted input from S into argument list ARG.  */
 extern int __vfscanf __P ((FILE *__s, __const char *__format,
 			   __gnuc_va_list __arg));
@@ -491,22 +489,22 @@ extern int vsscanf __P ((__const char *__s, __const char *__format,
 
 #ifdef	__OPTIMIZE__
 extern __inline int
-vfscanf (FILE *__s, const char *__fmt, __gnuc_va_list __arg)
+vfscanf (FILE *__s, const char *__fmt, __gnuc_va_list __arg) __THROW
 {
   return __vfscanf (__s, __fmt, __arg);
 }
 extern __inline int
-vscanf (const char *__fmt, __gnuc_va_list __arg)
+vscanf (const char *__fmt, __gnuc_va_list __arg) __THROW
 {
   return __vfscanf (stdin, __fmt, __arg);
 }
 extern __inline int
-vsscanf (const char *__s, const char *__fmt, __gnuc_va_list __arg)
+vsscanf (const char *__s, const char *__fmt, __gnuc_va_list __arg) __THROW
 {
   return __vsscanf (__s, __fmt, __arg);
 }
 #endif /* Optimizing.  */
-#endif /* Use GNU.  */
+#endif /* Use ISO C9x.  */
 
 
 /* This does actual reading when necessary, filling STREAM's
@@ -533,12 +531,32 @@ extern int getchar __P ((void));
 
 #ifdef	__OPTIMIZE__
 extern __inline int
-getchar (void)
+getchar (void) __THROW
 {
   return __getc (stdin);
 }
 #endif /* Optimizing.  */
 
+#if defined __USE_POSIX || defined __USE_MISC
+/* These are defined in POSIX.1:1996.  */
+extern int getc_unlocked __P ((FILE *__stream));
+extern int getchar_unlocked __P ((void));
+
+# ifdef __OPTIMIZE__
+extern __inline int
+getc_unlocked (FILE *__stream) __THROW
+{
+  return __getc (__stream);
+}
+
+extern __inline int
+getchar_unlocked (void) __THROW
+{
+  return __getc (stdin);
+}
+# endif /* Optimizing.  */
+#endif /* Use POSIX or MISC.  */
+
 
 /* Write a character to STREAM.  */
 extern int fputc __P ((int __c, FILE *__stream));
@@ -561,12 +579,45 @@ extern int putchar __P ((int __c));
 
 #ifdef __OPTIMIZE__
 extern __inline int
-putchar (int __c)
+putchar (int __c) __THROW
 {
   return __putc (__c, stdout);
 }
 #endif
 
+#ifdef __USE_MISC
+/* Faster version when locking is not necessary.  */
+extern int fputc_unlocked __P ((int __c, FILE *__stream));
+
+# ifdef __OPTIMIZE__
+extern __inline int
+fputc_unlocked (int __c, FILE *__stream) __THROW
+{
+  return __putc (__c, __stream);
+}
+# endif /* Optimizing.  */
+#endif /* Use MISC.  */
+
+#if defined __USE_POSIX || defined __USE_MISC
+/* These are defined in POSIX.1:1996.  */
+extern int putc_unlocked __P ((int __c, FILE *__stream));
+extern int putchar_unlocked __P ((int __c));
+
+# ifdef __OPTIMIZE__
+extern __inline int
+putc_unlocked (int __c, FILE *__stream) __THROW
+{
+  return __putc (__c, __stream);
+}
+
+extern __inline int
+putchar_unlocked (int __c) __THROW
+{
+  return __putc (__c, stdout);
+}
+# endif /* Optimizing.  */
+#endif /* Use POSIX or MISC.  */
+
 
 #if defined __USE_SVID || defined __USE_MISC
 /* Get a word (int) from STREAM.  */
@@ -581,6 +632,12 @@ extern int putw __P ((int __w, FILE *__stream));
 extern char *fgets __P ((char *__restrict __s, int __n,
 			 FILE *__restrict __stream));
 
+#ifdef __USE_GNU
+/* This function does the same as `fgets' but does not lock the stream.  */
+extern char *fgets_unlocked __P ((char *__restrict __s, int __n,
+				  FILE *__restrict __stream));
+#endif
+
 /* Get a newline-terminated string from stdin, removing the newline.
    DO NOT USE THIS FUNCTION!!  There is no limit on how much it will read.  */
 extern char *gets __P ((char *__s));
@@ -605,7 +662,7 @@ ssize_t getline __P ((char **__lineptr, size_t *__n, FILE *__stream));
 
 #ifdef	__OPTIMIZE__
 extern __inline ssize_t
-getline (char **__lineptr, size_t *__n, FILE *__stream)
+getline (char **__lineptr, size_t *__n, FILE *__stream) __THROW
 {
   return __getdelim (__lineptr, __n, '\n', __stream);
 }
@@ -616,6 +673,13 @@ getline (char **__lineptr, size_t *__n, FILE *__stream)
 /* Write a string to STREAM.  */
 extern int fputs __P ((__const char *__restrict __s,
 		       FILE *__restrict __stream));
+
+#ifdef __USE_GNU
+/* This function does the same as `fputs' but does not lock the stream.  */
+extern int fputs_unlocked __P ((__const char *__restrict __s,
+				FILE *__restrict __stream));
+#endif
+
 /* Write a string, followed by a newline, to stdout.  */
 extern int puts __P ((__const char *__s));
 
@@ -631,6 +695,15 @@ extern size_t fread __P ((__ptr_t __restrict __ptr, size_t __size,
 extern size_t fwrite __P ((__const __ptr_t __restrict __ptr, size_t __size,
 			   size_t __n, FILE *__restrict __s));
 
+#ifdef __USE_MISC
+/* Faster versions when locking is not necessary.  */
+extern size_t fread_unlocked __P ((void *__restrict __ptr, size_t __size,
+				   size_t __n, FILE *__restrict __stream));
+extern size_t fwrite_unlocked __P ((__const void *__restrict __ptr,
+				    size_t __size, size_t __n,
+				    FILE *__restrict __stream));
+#endif
+
 
 /* Seek to a certain position on STREAM.  */
 extern int fseek __P ((FILE *__stream, long int __off, int __whence));
@@ -657,6 +730,17 @@ extern int ferror __P ((FILE *__stream));
 #define	ferror(stream)	((stream)->__error != 0)
 #endif /* Optimizing.  */
 
+#ifdef __USE_MISC
+/* Faster versions when locking is not required.  */
+extern void clearerr_unlocked __P ((FILE *__stream));
+extern int feof_unlocked __P ((FILE *__stream));
+extern int ferror_unlocked __P ((FILE *__stream));
+
+# ifdef	__OPTIMIZE__
+#  define feof_unlocked(stream)		((stream)->__eof != 0)
+#  define ferror_unlocked(stream)	((stream)->__error != 0)
+# endif /* Optimizing.  */
+#endif
 
 /* Print a message describing the meaning of the value of errno.  */
 extern void perror __P ((__const char *__s));
@@ -676,6 +760,11 @@ extern const char *const _sys_errlist[];
 extern int fileno __P ((FILE *__stream));
 #endif /* Use POSIX.  */
 
+#ifdef __USE_MISC
+/* Faster version when locking is not required.  */
+extern int fileno_unlocked __P ((FILE *__stream));
+#endif
+
 
 #if (defined __USE_POSIX2 || defined __USE_SVID || defined __USE_BSD || \
      defined __USE_MISC)
@@ -714,6 +803,28 @@ extern int obstack_vprintf __P ((struct obstack *__obstack,
 #endif
 
 
+#if defined __USE_POSIX || defined __USE_MISC
+/* These are defined in POSIX.1:1996.  */
+
+/* Acquire ownership of STREAM.  */
+extern void flockfile __P ((FILE *__stream));
+
+/* Try to acquire ownership of STREAM but do not block if it is not
+   possible.  */
+extern int ftrylockfile __P ((FILE *__stream));
+
+/* Relinquish the ownership granted for STREAM.  */
+extern void funlockfile __P ((FILE *__stream));
+#endif /* POSIX || misc */
+
+#if defined __USE_XOPEN && !defined __USE_GNU
+/* The X/Open standard requires some functions and variables to be
+   declared here which do not belong into this header.  But we have to
+   follow.  In GNU mode we don't do this nonsense.  */
+# define __need_getopt
+# include <getopt.h>
+#endif
+
 __END_DECLS
 
 #endif /* <stdio.h> included.  */
