@@ -21,18 +21,22 @@
  */
 
 #ifndef	_STDLIB_H
-#define	_STDLIB_H	1
 
 #include <features.h>
 
 /* Get size_t, wchar_t and NULL from <stddef.h>.  */
-#define	__need_size_t
-#define	__need_wchar_t
-#define	__need_NULL
+#define		__need_size_t
+#ifndef __need_malloc_and_calloc
+# define	__need_wchar_t
+# define	__need_NULL
+#endif
 #include <stddef.h>
 
 __BEGIN_DECLS
 
+#ifndef __need_malloc_and_calloc
+#define	_STDLIB_H	1
+
 /* Returned by `div'.  */
 typedef struct
   {
@@ -49,7 +53,7 @@ typedef struct
 
 #ifdef __USE_ISOC9X
 /* Returned by `lldiv'.  */
-typedef struct
+__extension__ typedef struct
   {
     long long int quot;		/* Quotient.  */
     long long int rem;		/* Remainder.  */
@@ -81,14 +85,14 @@ extern long int atol __P ((__const char *__nptr));
 
 #if defined __USE_ISOC9X || (defined __GNUC__ && defined __USE_MISC)
 /* These functions will part of the standard C library in ISO C 9X.  */
-extern long long int atoll __P ((__const char *__nptr));
+__extension__ extern long long int atoll __P ((__const char *__nptr));
 #endif
 
 /* Convert a string to a floating-point number.  */
 extern double strtod __P ((__const char *__restrict __nptr,
 			   char **__restrict __endptr));
 
-#ifdef	__USE_GNU
+#ifdef	__USE_ISOC9X
 /* Likewise for `float' and `long double' sizes of floating-point numbers.  */
 extern float strtof __P ((__const char *__restrict __nptr,
 			  char **__restrict __endptr));
@@ -107,9 +111,11 @@ extern unsigned long int strtoul __P ((__const char *__restrict __nptr,
 
 #if defined __GNUC__ && defined __USE_BSD
 /* Convert a string to a quadword integer.  */
+__extension__
 extern long long int strtoq __P ((__const char *__restrict __nptr,
 				  char **__restrict __endptr, int __base));
 /* Convert a string to an unsigned quadword integer.  */
+__extension__
 extern unsigned long long int strtouq __P ((__const char *__restrict __nptr,
 					    char **__restrict __endptr,
 					    int __base));
@@ -119,9 +125,11 @@ extern unsigned long long int strtouq __P ((__const char *__restrict __nptr,
 /* These functions will part of the standard C library in ISO C 9X.  */
 
 /* Convert a string to a quadword integer.  */
+__extension__
 extern long long int strtoll __P ((__const char *__restrict __nptr,
 				   char **__restrict __endptr, int __base));
 /* Convert a string to an unsigned quadword integer.  */
+__extension__
 extern unsigned long long int strtoull __P ((__const char *__restrict __nptr,
 					     char **__restrict __endptr,
 					     int __base));
@@ -154,10 +162,12 @@ extern unsigned long int __strtoul_l __P ((__const char *__restrict __nptr,
 					   char **__restrict __endptr,
 					   int __base, __locale_t __loc));
 
+__extension__
 extern long long int __strtoll_l __P ((__const char *__restrict __nptr,
 				       char **__restrict __endptr, int __base,
 				       __locale_t __loc));
 
+__extension__
 extern unsigned long long int __strtoull_l __P ((__const char *__restrict
 						 __nptr,
 						 char **__restrict __endptr,
@@ -203,12 +213,14 @@ extern unsigned long int __strtoul_internal __P ((__const char *
 #endif
 #if defined __GNUC__ || defined __USE_ISOC9X
 # ifndef __strtoll_internal_defined
+__extension__
 extern long long int __strtoll_internal __P ((__const char *__restrict __nptr,
 					      char **__restrict __endptr,
 					      int __base, int __group));
 #  define __strtoll_internal_defined	1
 # endif
 # ifndef __strtoull_internal_defined
+__extension__
 extern unsigned long long int __strtoull_internal __P ((__const char *
 							__restrict __nptr,
 							char **
@@ -219,94 +231,95 @@ extern unsigned long long int __strtoull_internal __P ((__const char *
 # endif
 #endif /* GCC */
 
-#if defined __OPTIMIZE__ && __GNUC__ >= 2
+#if defined __OPTIMIZE__ && !defined __OPTIMIZE_SIZE__ \
+    && defined __USE_EXTERN_INLINES
 /* Define inline functions which call the internal entry points.  */
 
 extern __inline double
-strtod (__const char *__restrict __nptr, char **__restrict __endptr)
+strtod (__const char *__restrict __nptr, char **__restrict __endptr) __THROW
 {
   return __strtod_internal (__nptr, __endptr, 0);
 }
 extern __inline long int
 strtol (__const char *__restrict __nptr, char **__restrict __endptr,
-	int __base)
+	int __base) __THROW
 {
   return __strtol_internal (__nptr, __endptr, __base, 0);
 }
 extern __inline unsigned long int
 strtoul (__const char *__restrict __nptr, char **__restrict __endptr,
-	 int __base)
+	 int __base) __THROW
 {
   return __strtoul_internal (__nptr, __endptr, __base, 0);
 }
 
-# ifdef __USE_GNU
+# ifdef __USE_ISOC9X
 extern __inline float
-strtof (__const char *__restrict __nptr, char **__restrict __endptr)
+strtof (__const char *__restrict __nptr, char **__restrict __endptr) __THROW
 {
   return __strtof_internal (__nptr, __endptr, 0);
 }
 extern __inline __long_double_t
-strtold (__const char *__restrict __nptr, char **__restrict __endptr)
+strtold (__const char *__restrict __nptr, char **__restrict __endptr) __THROW
 {
   return __strtold_internal (__nptr, __endptr, 0);
 }
 # endif
 
 # ifdef __USE_BSD
-extern __inline long long int
+__extension__ extern __inline long long int
 strtoq (__const char *__restrict __nptr, char **__restrict __endptr,
-	int __base)
+	int __base) __THROW
 {
   return __strtoll_internal (__nptr, __endptr, __base, 0);
 }
-extern __inline unsigned long long int
+__extension__ extern __inline unsigned long long int
 strtouq (__const char *__restrict __nptr, char **__restrict __endptr,
-	 int __base)
+	 int __base) __THROW
 {
   return __strtoull_internal (__nptr, __endptr, __base, 0);
 }
 # endif
 
 # if defined __USE_MISC || defined __USE_ISOC9X
-extern __inline long long int
+__extension__ extern __inline long long int
 strtoll (__const char *__restrict __nptr, char **__restrict __endptr,
-	 int __base)
+	 int __base) __THROW
 {
   return __strtoll_internal (__nptr, __endptr, __base, 0);
 }
-extern __inline unsigned long long int
+__extension__ extern __inline unsigned long long int
 strtoull (__const char * __restrict __nptr, char **__restrict __endptr,
-	  int __base)
+	  int __base) __THROW
 {
   return __strtoull_internal (__nptr, __endptr, __base, 0);
 }
 # endif
 
 extern __inline double
-atof (__const char *__nptr)
+atof (__const char *__nptr) __THROW
 {
   return strtod (__nptr, (char **) NULL);
 }
 extern __inline int
-atoi (__const char *__nptr)
+atoi (__const char *__nptr) __THROW
 {
   return (int) strtol (__nptr, (char **) NULL, 10);
 }
 extern __inline long int
-atol (__const char *__nptr)
+atol (__const char *__nptr) __THROW
 {
   return strtol (__nptr, (char **) NULL, 10);
 }
 
 # if defined __USE_MISC || defined __USE_ISOC9X
-extern __inline long long int
-atoll (__const char *__nptr)
+__extension__ extern __inline long long int
+atoll (__const char *__nptr) __THROW
 {
   return strtoll (__nptr, (char **) NULL, 10);
 }
 # endif
-#endif /* Optimizing GCC >=2.  */
+#endif /* Optimizing and Inlining.  */
 
 
 #if defined __USE_SVID || defined __USE_XOPEN_EXTENDED
@@ -326,25 +339,20 @@ extern long int a64l __P ((__const char *__s));
    The `rand' and `srand' functions are required by the ANSI standard.
    We provide both interfaces to the same random number generator.  */
 /* Return a random long integer between 0 and RAND_MAX inclusive.  */
-extern int32_t __random __P ((void));
 extern int32_t random __P ((void));
 
 /* Seed the random number generator with the given number.  */
-extern void __srandom __P ((unsigned int __seed));
 extern void srandom __P ((unsigned int __seed));
 
 /* Initialize the random number generator to use state buffer STATEBUF,
    of length STATELEN, and seed it with SEED.  Optimal lengths are 8, 16,
    32, 64, 128 and 256, the bigger the better; values less than 8 will
    cause an error and values greater than 256 will be rounded down.  */
-extern __ptr_t __initstate __P ((unsigned int __seed, __ptr_t __statebuf,
-				 size_t __statelen));
 extern __ptr_t initstate __P ((unsigned int __seed, __ptr_t __statebuf,
 			       size_t __statelen));
 
 /* Switch the random number generator to state buffer STATEBUF,
    which should have been previously initialized by `initstate'.  */
-extern __ptr_t __setstate __P ((__ptr_t __statebuf));
 extern __ptr_t setstate __P ((__ptr_t __statebuf));
 
 
@@ -364,18 +372,13 @@ struct random_data
     int32_t *end_ptr;		/* Pointer behind state table.  */
   };
 
-extern int __random_r __P ((struct random_data *__buf, int32_t *__result));
 extern int random_r __P ((struct random_data *__buf, int32_t *__result));
 
-extern int __srandom_r __P ((unsigned int __seed, struct random_data *__buf));
 extern int srandom_r __P ((unsigned int __seed, struct random_data *__buf));
 
-extern int __initstate_r __P ((unsigned int __seed, __ptr_t __statebuf,
-			       size_t __statelen, struct random_data *__buf));
 extern int initstate_r __P ((unsigned int __seed, __ptr_t __statebuf,
 			     size_t __statelen, struct random_data *__buf));
 
-extern int __setstate_r __P ((__ptr_t __statebuf, struct random_data *__buf));
 extern int setstate_r __P ((__ptr_t __statebuf, struct random_data *__buf));
 # endif	/* Use misc.  */
 #endif	/* Use SVID || extended X/Open.  */
@@ -386,9 +389,8 @@ extern int rand __P ((void));
 /* Seed the random number generator with the given number.  */
 extern void srand __P ((unsigned int __seed));
 
-#ifdef __USE_MISC
+#ifdef __USE_POSIX
 /* Reentrant interface according to POSIX.1.  */
-extern int __rand_r __P ((unsigned int *__seed));
 extern int rand_r __P ((unsigned int *__seed));
 #endif
 
@@ -426,56 +428,44 @@ struct drand48_data
 # ifdef __USE_MISC
 /* Return non-negative, double-precision floating-point value in [0.0,1.0).  */
 extern int drand48_r __P ((struct drand48_data *__buffer, double *__result));
-extern int __erand48_r __P ((unsigned short int __xsubi[3],
-			     struct drand48_data *__buffer, double *__result));
 extern int erand48_r __P ((unsigned short int __xsubi[3],
 			   struct drand48_data *__buffer, double *__result));
 
 /* Return non-negative, long integer in [0,2^31).  */
 extern int lrand48_r __P ((struct drand48_data *__buffer, long int *__result));
-extern int __nrand48_r __P ((unsigned short int __xsubi[3],
-			     struct drand48_data *__buffer,
-			     long int *__result));
 extern int nrand48_r __P ((unsigned short int __xsubi[3],
 			   struct drand48_data *__buffer, long int *__result));
 
 /* Return signed, long integers in [-2^31,2^31).  */
 extern int mrand48_r __P ((struct drand48_data *__buffer, long int *__result));
-extern int __jrand48_r __P ((unsigned short int __xsubi[3],
-			     struct drand48_data *__buffer,
-			     long int *__result));
 extern int jrand48_r __P ((unsigned short int __xsubi[3],
 			   struct drand48_data *__buffer, long int *__result));
 
 /* Seed random number generator.  */
-extern int __srand48_r __P ((long int __seedval,
-			     struct drand48_data *__buffer));
 extern int srand48_r __P ((long int __seedval, struct drand48_data *__buffer));
 
-extern int __seed48_r __P ((unsigned short int __seed16v[3],
-			    struct drand48_data *__buffer));
 extern int seed48_r __P ((unsigned short int __seed16v[3],
 			  struct drand48_data *__buffer));
 
-extern int __lcong48_r __P ((unsigned short int __param[7],
-			     struct drand48_data *__buffer));
 extern int lcong48_r __P ((unsigned short int __param[7],
 			   struct drand48_data *__buffer));
 # endif	/* Use misc.  */
-
-/* Internal function to compute next state of the generator.  */
-extern int __drand48_iterate __P ((unsigned short int __xsubi[3],
-				   struct drand48_data *__buffer));
 #endif	/* Use SVID or X/Open.  */
 
+#endif /* don't just need malloc and calloc */
 
+#ifndef __malloc_and_calloc_defined
+#define __malloc_and_calloc_defined
 /* Allocate SIZE bytes of memory.  */
 extern __ptr_t malloc __P ((size_t __size));
+/* Allocate NMEMB elements of SIZE bytes each, all initialized to 0.  */
+extern __ptr_t calloc __P ((size_t __nmemb, size_t __size));
+#endif
+
+#ifndef __need_malloc_and_calloc
 /* Re-allocate the previously allocated block
    in __ptr_t, making the new block SIZE bytes long.  */
 extern __ptr_t realloc __P ((__ptr_t __ptr, size_t __size));
-/* Allocate NMEMB elements of SIZE bytes each, all initialized to 0.  */
-extern __ptr_t calloc __P ((size_t __nmemb, size_t __size));
 /* Free a block allocated by `malloc', `realloc' or `calloc'.  */
 extern void free __P ((__ptr_t __ptr));
 
@@ -544,7 +534,6 @@ extern void unsetenv __P ((__const char *__name));
 /* The `clearenv' was planned to be added to POSIX.1 but probably
    never made it.  Nevertheless the POSIX.9 standard (POSIX bindings
    for Fortran 77) requires this function.  */
-extern int __clearenv __P ((void));
 extern int clearenv __P ((void));
 #endif
 
@@ -573,7 +562,6 @@ extern int system __P ((__const char *__command));
 /* Return a malloc'd string containing the canonical absolute name of the
    named file.  The last file name component need not exist, and may be a
    symlink to a nonexistent file.  */
-extern char *__canonicalize_file_name __P ((__const char *__name));
 extern char *canonicalize_file_name __P ((__const char *__name));
 #endif
 
@@ -584,7 +572,6 @@ extern char *canonicalize_file_name __P ((__const char *__name));
    name is PATH_MAX chars or more, returns null with `errno' set to
    ENAMETOOLONG; if the name fits in fewer than PATH_MAX chars, returns the
    name in RESOLVED.  */
-extern char *__realpath __P ((__const char *__name, char *__resolved));
 extern char *realpath __P ((__const char *__name, char *__resolved));
 #endif
 
@@ -592,7 +579,7 @@ extern char *realpath __P ((__const char *__name, char *__resolved));
 /* Shorthand for type of comparison functions.  */
 #ifndef __COMPAR_FN_T
 # define __COMPAR_FN_T
-typedef int (*__compar_fn_t) __P ((__const __ptr_t, __const __ptr_t));
+typedef int (*__compar_fn_t) __PMT ((__const __ptr_t, __const __ptr_t));
 
 # ifdef	__USE_GNU
 typedef __compar_fn_t comparison_fn_t;
@@ -601,21 +588,21 @@ typedef __compar_fn_t comparison_fn_t;
 
 /* Do a binary search for KEY in BASE, which consists of NMEMB elements
    of SIZE bytes each, using COMPAR to perform the comparisons.  */
-extern __ptr_t bsearch __P ((__const __ptr_t __key, __const __ptr_t __base,
-			     size_t __nmemb, size_t __size,
-			     __compar_fn_t __compar));
+extern __ptr_t bsearch __PMT ((__const __ptr_t __key, __const __ptr_t __base,
+			       size_t __nmemb, size_t __size,
+			       __compar_fn_t __compar));
 
 /* Sort NMEMB elements of BASE, of SIZE bytes each,
    using COMPAR to perform the comparisons.  */
-extern void qsort __P ((__ptr_t __base, size_t __nmemb, size_t __size,
-			__compar_fn_t __compar));
+extern void qsort __PMT ((__ptr_t __base, size_t __nmemb, size_t __size,
+			  __compar_fn_t __compar));
 
 
 /* Return the absolute value of X.  */
 extern int abs __P ((int __x)) __attribute__ ((__const__));
 extern long int labs __P ((long int __x)) __attribute__ ((__const__));
 #ifdef __USE_ISOC9X
-extern long long int llabs __P ((long long int __x))
+__extension__ extern long long int llabs __P ((long long int __x))
      __attribute__ ((__const__));
 #endif
 
@@ -627,7 +614,8 @@ extern div_t div __P ((int __numer, int __denom)) __attribute__ ((__const__));
 extern ldiv_t ldiv __P ((long int __numer, long int __denom))
      __attribute__ ((__const__));
 #ifdef __USE_ISOC9X
-extern lldiv_t lldiv __P ((long long int __numer, long long int __denom))
+__extension__ extern lldiv_t lldiv __P ((long long int __numer,
+					 long long int __denom))
      __attribute__ ((__const__));
 #endif
 
@@ -745,14 +733,15 @@ extern char *ptsname __P ((int __fd));
 /* Store at most BUFLEN characters of the pathname of the slave pseudo
    terminal associated with the master FD is open on in BUF.
    Return 0 on success, otherwise an error number.  */
-extern int __ptsname_r __P ((int __fd, char *__buf, size_t __buflen));
 extern int ptsname_r __P ((int __fd, char *__buf, size_t __buflen));
 
 /* Open a master pseudo terminal and return its file descriptor.  */
-extern int __getpt __P ((void));
 extern int getpt __P ((void));
 #endif
 
+#endif /* don't just need malloc and calloc */
+#undef __need_malloc_and_calloc
+
 __END_DECLS
 
 #endif /* stdlib.h  */
